/*
 *
 *                         DES SOFTWARE PACKAGE
 *                             Version 2.2
 *
 *                                        _
 * Copyright (c) 1990,1991,1992,1993 Stig Ostholm.
 * All Rights Reserved
 *
 *
 * The author takes no responsibility of actions caused by the use of this
 * software package and does not guarantee the correctness of the functions.
 *
 * This software package may be freely distributed for non-commercial purpose
 * as long as the copyright notice is kept. Any changes made should be
 * accompanied by a comment indicating who made the change, when it was made
 * and what was changed.
 *
 * This software package, or any parts of it, may not be used or in any way
 * re-distributed for commercial purpose without the authors permission.
 * The author keeps the right to decide between of what is commercial and
 * what is non-commercial purpose.
 *
 * Restrictions due to national laws governing the use, import or export of
 * cryptographic software is the responsibility of the software user/importer/
 * exporter to follow.
 *
 *
 *                                              _
 *                                         Stig Ostholm
 *                                         Chalmers University of Technology
 *                                         Department of Computer Engineering
 *                                         S-412 96 Gothenburg
 *                                         Sweden
 *                                         ----------------------------------
 *                                         Email: ostholm@ce.chalmers.se
 *                                         Phone: +46 31 772 1703
 *                                         Fax:   +46 31 772 3663
 */

#include	"des.h"
#ifndef DEA3_TEST
# include	"dea3_conf.h"
#endif  /* DEA3_TEST */
#ifndef USE_DES_DEA
# include	"dea.h"
#endif  /* USE_DES_DEA */
#include	"version.h"

/*
 * des_dea3b
 *
 *	Trippel DES encryption alogrithm as proposed by
 *	various sources (a generalization of ANSI X9.17-1985.).
 *
 *	E = Encrypt DES.
 *	D = Dencrypt DES.
 *	E3 = ANSI Encrypt Trippel DES.
 *	D3 = ANSI Dencrypt Trippel DES.
 *	E3b = Encrypt Trippel DES.
 *	D3b = Dencrypt Trippel DES.
 *
 *	E3b({K1, K2, K3}, x) = E(K3, D(K2, E(K1, x)))
 *	D3b({K1, K2, K3}, x) = D(K3, E(K2, D(K1, x)))
 *
 *	Note: K1 = K2 = K3 -> E3b = E, D3b = D.
 *	      K1 = K3 -> E3b = E3, D3B = D3
 *
 *	`input' and `output' may be overlapping.
 */


int	des_dea3b(
#ifdef __STDC__
	des_cblock		*input,
	des_cblock		*output,
	des_key_schedule	schedule,
	int			encrypt)
#else
	input, output, schedule, encrypt)
des_cblock		*input;
des_cblock		*output;
des_key_schedule	schedule;
int			encrypt;
#endif
{
	register des_key_schedule	*schedule1, *schedule2, *schedule3;
#ifndef USE_DES_DEA
	register unsigned long		l, r;
#ifdef IP_DATA
	IP_DATA;
#endif /* IP_DATA */
#ifdef CRYPT_DATA
	CRYPT_DATA;
#endif /* CRYPT_DATA */
#ifdef IPinv_DATA
	IPinv_DATA;
#endif /* IPinv_DATA */
#endif  /* USE_DES_DEA */


	schedule1 = (des_key_schedule *) schedule;
	schedule2 = schedule1 + 1;
	schedule3 = schedule2 + 1;

#ifndef USE_DES_DEA
	/* IP - Initial Permutaion */
	IP(l, r, (*input));
#endif  /* !USE_DES_DEA */

	if (encrypt) {
#ifdef USE_DES_DEA
		(void) des_dea(input, output, *schedule1, DES_ENCRYPT);
		(void) des_dea(output, output, *schedule2, DES_DECRYPT);
		(void) des_dea(output, output, *schedule3, DES_ENCRYPT);
#else  /* USE_DES_DEA */
		ENCRYPT(l, r, *schedule1)
		/* `IP' and `IPinv' eliminates eachother */
		DECRYPT(r, l, *schedule2)
		/* `IP' and `IPinv' eliminates eachother */
		ENCRYPT(l, r, *schedule3)
#endif /* USE_DES_DEA */
	} else {
#ifdef USE_DES_DEA
		(void) des_dea(input, output, *schedule3, DES_DECRYPT);
		(void) des_dea(output, output, *schedule2, DES_ENCRYPT);
		(void) des_dea(output, output, *schedule1, DES_DECRYPT);
#else  /* USE_DES_DEA */
		DECRYPT(l, r, *schedule3)
		/* `IP' and `IPinv' eliminates eachother */
		ENCRYPT(r, l, *schedule2)
		/* `IP' and `IPinv' eliminates eachother */
		DECRYPT(l, r, *schedule1)
#endif /* USE_DES_DEA */
	}

#ifndef USE_DES_DEA
	/* IPinv - Inverse of Initial Permutaion IP */
	IPinv((*output), r, l);
#endif  /* !USE_DES_DEA */

	return 0;
}
